--- File-Filter
-- @author Enno Sylvester
-- @copyright 2019, 2020
-- @module eatglobal.modutils

local historie = [[
	Version 1.7:
	 - Added: Behandlung weiterer Usereinstellungen hinzugefügt
]]

local verionsData = {
	majorVersion = 1,
	minorVersion = 7,
}

local modutils = {
	version = string.format("%d.%d", verionsData.majorVersion, verionsData.minorVersion),
}

local function init ()
	-- globale Daten
	if (type(eat1963_global_data) ~= "table") then
		eat1963_global_data = {}
	end
	-- modutils
	if (type(eat1963_global_data.modutils) ~= "table") then
		eat1963_global_data.modutils = {
			modelHandler = {
				modelHandlerRegistered = false,
				models = {},
			},
			modData = {},
			version = verionsData,
		}
	else
		local version = modutils.eatglobal.utils.getValue(eat1963_global_data.modutils.version, {majorVersion = 0, minorVersion = 0})
		if ((version.majorVersion < verionsData.majorVersion) or (version.minorVersion < verionsData.minorVersion)) then
			eat1963_global_data.modutils.modData = {}
			eat1963_global_data.modutils.version = verionsData
		end
	end
end

-- Modifier
local function modelHandler(fileName, data, key)
	local function findObject(fileName)
		local models = eat1963_global_data.modutils.modelHandler.models
		for model in modutils.eatglobal.utils.ipairs(models) do
			if string.ends(fileName, model.fileName) then
				return model
			end
		end
		return nil
	end
	
	local objectData = findObject(fileName)
	if (type(objectData) == "table") then
		return objectData.fun(fileName, data, key, modutils.eatglobal.mod.getSubKey(key), objectData.funData)
	end
	
	return data
end

local function doHandleObject (data, fun, funData)
	local handler = eat1963_global_data.modutils.modelHandler
	local function putData(name)
		handler.models[#handler.models + 1] = {
			fileName = name,
			fun = fun,
			funData = funData,
		}
	end
	--------------------------
	if (not handler.modelHandlerRegistered) then
		modutils.eatglobal.mod.addModifier("all", modelHandler)
		handler.modelHandlerRegistered = true
	end
	
	if (type(data) == "table") then
		for name in modutils.eatglobal.utils.ipairs(data) do
			if modutils.eatglobal.utils.isValidString(name) then
				putData(name)
			end
		end
	elseif modutils.eatglobal.utils.isValidString(data) then
		putData(data)
	end
end

local function getDefaultSettings(modData)
	local function hasFileOrFiles(data)
		return (type(data) == "table") or modutils.eatglobal.utils.isValidString(data)
	end
	local function makeSettingsEntry(val, result, order)
		result[val.key] = {
			type = type(val.default),
			name = _(val.name),
			description = _(val.description),
			values = val.values,
			default = val.default,
			multiSelect = val.multiSelect,
			order = order,
			image = val.icon,
		}
		return order + 1
	end
	--------------------------------------------------
	local getValue = modutils.eatglobal.utils.getValue
	local result = {}
	local order = 1
	local defaultSettings = modData.defaultSettings
	local hasVorspannFiles = false
	local hasFakeFiles = false
	
	-- data checking
	modData = getValue(modData, {})
	modData.defaultSettings = getValue(modData.defaultSettings, {})
	modData.defaultSettings.files = getValue(modData.defaultSettings.files, {})
	
	-- data handling
	--	files
	local set = eatset.new(defaultSettings.files)
	for val in set:_pairs() do
		order = makeSettingsEntry(val, result, order)
		if (not hasVorspannFiles) then
			hasVorspannFiles = hasFileOrFiles(val.vorspannFile)
		end
		if (not hasFakeFiles) then
			hasFakeFiles = hasFileOrFiles(val.fakeFile)
		end
	end

	--[[
	for i = 1, #defaultSettings.files do
		local item = defaultSettings.files[i]
		local itemValue = item.default or false
		result[item.key] = {
			type = type(itemValue),
			default = itemValue,
			order = order,
			name = _(item.name),
			description = _(item.description),
			image = item.icon,
		}
		if (not hasVorspannFiles) then
			hasVorspannFiles = hasFileOrFiles(item.vorspannFile)
		end
		if (not hasFakeFiles) then
			hasFakeFiles = hasFileOrFiles(item.fakeFile)
		end
		
		order = order + 1
	end
	]]
	
	if (hasVorspannFiles) then
		local item = defaultSettings.vorspann or {}
		local key = item.key or "vorspann"
		result[key] = {
			type = item.type or "boolean",
			default = item.default or false,
			order = order,
			name = _(item.name or "Vorspann"),
			description = _(item.description or ""),
			image = item.icon or "",
		}
		order = order + 1
		modData.defaultSettings.vorspannKey = key
	else
		modData.defaultSettings.vorspannKey = ""
	end
	
	if (hasFakeFiles) then
		local item = defaultSettings.fake or {}
		local key = item.key or "fake"
		result[key] = {
			type = item.type or "boolean",
			default = item.default or false,
			order = order,
			name = _(item.name or "Fake"),
			description = _(item.description or ""),
			image = item.icon or "",
		}
		order = order + 1
		modData.defaultSettings.fakeKey = key
	else
		modData.defaultSettings.fakeKey = ""
	end
	
	-- Start- und Endjahr
	if (type(defaultSettings.disableStartYear) == "table") then
		local item = defaultSettings.disableStartYear or {}
		local key = item.key or "disableStartYear"
		result[key] = {
			type = "boolean",
			default = item.default or false,
			order = order,
			name = _(item.name or "Disable Startyear"),
			description = _(item.description or ""),
			image = item.icon or "",
		}
		order = order + 1
		modData.defaultSettings.disableStartYearKey = key
	else
		modData.defaultSettings.disableStartYearKey = ""
	end
	
	if (type(defaultSettings.disableEndYear) == "table") then
		local item = defaultSettings.disableEndYear or {}
		local key = item.key or "disableEndYear"
		result[key] = {
			type = "boolean",
			default = item.default or false,
			order = order,
			name = _(item.name or "Disable Endyear"),
			description = _(item.description or ""),
			image = item.icon or "",
		}
		order = order + 1
		modData.defaultSettings.disableEndYearKey = key
	else
		modData.defaultSettings.disableEndYearKey = ""
	end
	
	-- andere Usereinstellungen
	if (type(defaultSettings.normalSettings) == "table") then
		local set = eatset.new(defaultSettings.normalSettings)
		for val in set:_pairs() do
			order = makeSettingsEntry(val, result, order)
		end
	end
	
	return result
end

local function doHandleStartAndEndYear (fileName, data, key, subKey, funData)
	modutils.logFn("doHandleStartAndEndYear | fileName = ", fileName)
	local function __do(key)
		if funData.handleStartYear then
			key.yearFrom = 0
		end
		if funData.handleEndYear then
			key.yearTo = 0
		end
	end
	if (subKey ~= "loadSoundSet") then
		if ((subKey == "loadModel") and (type(data.metadata) == "table") and (type(data.metadata.availability) == "table")) then
			__do(data.metadata.availability)
		elseif ((subKey == "loadConstruction") and (type(data.availability) == "table")) then
			__do(data.availability)
		else
			__do(data)
		end
	end
	
	return data
end

local function create(modId, modData)
	local doHideObject = modutils.eatglobal.mod.hideObject
	--- Hilfsmethode(n)
	-- Usersetting --> Modelle ein-/ausblenden
	local function handleUserSettings(modData, userSettings)
		defaultSettings = modData.defaultSettings
		for i = 1, #defaultSettings.files do
			local enableModel = userSettings[defaultSettings.files[i].key]
			if (type(enableModel) == "boolean") then
				-- Modell
				if not enableModel then
					doHideObject(defaultSettings.files[i].modelFile)
				end
				-- Vorspann
				--if (not enableModel) or (not defaultSettings.enableVorspann) then
				if (not enableModel) or (not userSettings[defaultSettings.vorspannKey]) then
					doHideObject(defaultSettings.files[i].vorspannFile)
				end
				-- Fake
				--if (not enableModel) or (not userSettings[defaultSettings.enableFake) then
				if (not enableModel) or (not userSettings[defaultSettings.fakeKey]) then
					doHideObject(defaultSettings.files[i].fakeFile)
				end
			end
			-- Start- und Endjahr
			local yearData = {
				handleStartYear = userSettings[defaultSettings.disableStartYearKey],
				handleEndYear = userSettings[defaultSettings.disableEndYearKey],
			}
			if ((yearData.handleStartYear == true) or (yearData.handleEndYear == true)) then
				doHandleObject(defaultSettings.files[i].modelFile, doHandleStartAndEndYear, yearData)
				doHandleObject(defaultSettings.files[i].vorspannFile, doHandleStartAndEndYear, yearData)
				doHandleObject(defaultSettings.files[i].fakeFile, doHandleStartAndEndYear, yearData)
			end
		end
	end
	
	--- HIER geht's los!!!!
	init()
	if modutils.eatglobal.utils.isValidString(modId) then
		if (type(eat1963_global_data.modutils.modData[modId]) ~= "table") then
			modData.general = modutils.eatglobal.utils.getValue(modData.general, {})
			local userSettings = modutils.eatglobal.usersettings.create(modId, getDefaultSettings(modData))
			-- Benutzereinstellungen anwenden
			handleUserSettings(modData, userSettings)
			doHideObject(modData.general.hideFiles, true)
			eat1963_global_data.modutils.modData[modId] = modData
		end
		
		return true
	else
		return false
	end
end

modutils.userSettings = {
	create = function(modId, modData) return create(modId, modData) end,
	getDefaultSettings = function(modData) return getDefaultSettings(modData) end,
	getUserSettings = function(modId) return modutils.eatglobal.usersettings.getSettings(modId) end,
}

return modutils